@@ -12,7 +12,11 @@ from mypy.plugin import ClassDefContext, SemanticAnalyzerPluginInterface
 from mypy.plugins.common import (
     add_method, _get_decorator_bool_argument, deserialize_and_fixup_type,
 )
-from mypy.types import Type, Instance, NoneType, TypeVarDef, TypeVarType, get_proper_type
+from mypy.typeops import map_type_from_supertype
+from mypy.types import (
+    Type, Instance, NoneType, TypeVarDef, TypeVarType, CallableType,
+    get_proper_type
+)
 from mypy.server.trigger import make_wildcard_trigger
 
 # The set of decorators that generate dataclasses.
@@ -34,6 +38,7 @@ class DataclassAttribute:
             line: int,
             column: int,
             type: Optional[Type],
+            info: TypeInfo,
     ) -> None:
         self.name = name
         self.is_in_init = is_in_init
@@ -42,6 +47,7 @@ class DataclassAttribute:
         self.line = line
         self.column = column
         self.type = type
+        self.info = info
 
     def to_argument(self) -> Argument:
         return Argument(
@@ -72,7 +78,15 @@ class DataclassAttribute:
     ) -> 'DataclassAttribute':
         data = data.copy()
         typ = deserialize_and_fixup_type(data.pop('type'), api)
-        return cls(type=typ, **data)
+        return cls(type=typ, info=info, **data)
+
+    def expand_typevar_from_subtype(self, sub_type: TypeInfo) -> None:
+        """Expands type vars in the context of a subtype when an attribute is inherited
+        from a generic super type."""
+        if not isinstance(self.type, TypeVarType):
+            return
+
+        self.type = map_type_from_supertype(self.type, sub_type, self.info)
 
 
 class DataclassTransformer:
@@ -123,27 +137,6 @@ class DataclassTransformer:
                                          [], obj_type)
             info.names[SELF_TVAR_NAME] = SymbolTableNode(MDEF, self_tvar_expr)
 
-        # Add an eq method, but only if the class doesn't already have one.
-        if decorator_arguments['eq'] and info.get('__eq__') is None:
-            for method_name in ['__eq__', '__ne__']:
-                # The TVar is used to enforce that "other" must have
-                # the same type as self (covariant).  Note the
-                # "self_type" parameter to add_method.
-                obj_type = ctx.api.named_type('__builtins__.object')
-                cmp_tvar_def = TypeVarDef(SELF_TVAR_NAME, info.fullname + '.' + SELF_TVAR_NAME,
-                                          -1, [], obj_type)
-                cmp_other_type = TypeVarType(cmp_tvar_def)
-                cmp_return_type = ctx.api.named_type('__builtins__.bool')
-
-                add_method(
-                    ctx,
-                    method_name,
-                    args=[Argument(Var('other', cmp_other_type), cmp_other_type, None, ARG_POS)],
-                    return_type=cmp_return_type,
-                    self_type=cmp_other_type,
-                    tvar_def=cmp_tvar_def,
-                )
-
         # Add <, >, <=, >=, but only if the class has an eq method.
         if decorator_arguments['order']:
             if not decorator_arguments['eq']:
@@ -180,6 +173,8 @@ class DataclassTransformer:
 
         if decorator_arguments['frozen']:
             self._freeze(attributes)
+        else:
+            self._propertize_callables(attributes)
 
         self.reset_init_only_vars(info, attributes)
 
@@ -288,6 +283,7 @@ class DataclassTransformer:
                 line=stmt.line,
                 column=stmt.column,
                 type=sym.type,
+                info=cls.info,
             ))
 
         # Next, collect attributes belonging to any class in the MRO
@@ -308,6 +304,7 @@ class DataclassTransformer:
                 name = data['name']  # type: str
                 if name not in known_attrs:
                     attr = DataclassAttribute.deserialize(info, data, ctx.api)
+                    attr.expand_typevar_from_subtype(ctx.cls.info)
                     known_attrs.add(name)
                     super_attrs.append(attr)
                 elif all_attrs:
@@ -361,6 +358,24 @@ class DataclassTransformer:
                 var._fullname = info.fullname + '.' + var.name
                 info.names[var.name] = SymbolTableNode(MDEF, var)
 
+    def _propertize_callables(self, attributes: List[DataclassAttribute]) -> None:
+        """Converts all attributes with callable types to @property methods.
+
+        This avoids the typechecker getting confused and thinking that
+        `my_dataclass_instance.callable_attr(foo)` is going to receive a
+        `self` argument (it is not).
+
+        """
+        info = self._ctx.cls.info
+        for attr in attributes:
+            if isinstance(get_proper_type(attr.type), CallableType):
+                var = attr.to_var()
+                var.info = info
+                var.is_property = True
+                var.is_settable_property = True
+                var._fullname = info.fullname + '.' + var.name
+                info.names[var.name] = SymbolTableNode(MDEF, var)
+
 
 def dataclass_class_maker_callback(ctx: ClassDefContext) -> None:
     """Hooks into the class typechecking process to add support for dataclasses.
