@@ -5,13 +5,12 @@ import os.path
 from mypy import build
 from mypy.modulefinder import BuildSource
 from mypy.test.helpers import (
-    assert_string_arrays_equal, testfile_pyversion, normalize_error_messages
+    assert_string_arrays_equal, normalize_error_messages, parse_options
 )
 from mypy.test.data import DataDrivenTestCase, DataSuite
 from mypy.test.config import test_temp_dir
 from mypy.test.visitors import TypeAssertTransformVisitor
 from mypy.errors import CompileError
-from mypy.options import Options
 
 
 class TransformSuite(DataSuite):
@@ -36,11 +35,10 @@ def test_transform(testcase: DataDrivenTestCase) -> None:
 
     try:
         src = '\n'.join(testcase.input)
-        options = Options()
+        options = parse_options(src, testcase, 1)
         options.use_builtins_fixtures = True
         options.semantic_analysis_only = True
         options.show_traceback = True
-        options.python_version = testfile_pyversion(testcase.file)
         result = build.build(sources=[BuildSource('main', None, src)],
                              options=options,
                              alt_lib_path=test_temp_dir)
@@ -62,6 +60,7 @@ def test_transform(testcase: DataDrivenTestCase) -> None:
                     and not os.path.splitext(
                         os.path.basename(f.path))[0].endswith('_')):
                 t = TypeAssertTransformVisitor()
+                t.test_only = True
                 f = t.mypyfile(f)
                 a += str(f).split('\n')
     except CompileError as e:
