@@ -18,7 +18,7 @@ from mypy.types import TupleType, get_proper_type, Instance
 
 from mypyc.common import MAX_SHORT_INT
 from mypyc.ir.ops import (
-    Value, Register, TupleGet, TupleSet, BasicBlock, Assign, LoadAddress
+    Value, Register, TupleGet, TupleSet, BasicBlock, Assign, LoadAddress, RaiseStandardError
 )
 from mypyc.ir.rtypes import (
     RTuple, object_rprimitive, is_none_rprimitive, int_rprimitive, is_int_rprimitive
@@ -30,19 +30,26 @@ from mypyc.primitives.misc_ops import new_slice_op, ellipsis_op, type_op, get_mo
 from mypyc.primitives.list_ops import list_append_op, list_extend_op, list_slice_op
 from mypyc.primitives.tuple_ops import list_tuple_op, tuple_slice_op
 from mypyc.primitives.dict_ops import dict_new_op, dict_set_item_op, dict_get_item_op
-from mypyc.primitives.set_ops import new_set_op, set_add_op, set_update_op
+from mypyc.primitives.set_ops import set_add_op, set_update_op
 from mypyc.primitives.str_ops import str_slice_op
 from mypyc.primitives.int_ops import int_comparison_op_mapping
 from mypyc.irbuild.specialize import specializers
 from mypyc.irbuild.builder import IRBuilder
-from mypyc.irbuild.for_helpers import translate_list_comprehension, comprehension_helper
+from mypyc.irbuild.for_helpers import (
+    translate_list_comprehension, translate_set_comprehension,
+    comprehension_helper
+)
 
 
 # Name and attribute references
 
 
 def transform_name_expr(builder: IRBuilder, expr: NameExpr) -> Value:
-    assert expr.node, "RefExpr not resolved"
+    if expr.node is None:
+        builder.add(RaiseStandardError(RaiseStandardError.RUNTIME_ERROR,
+                                       "mypyc internal error: should be unreachable",
+                                       expr.line))
+        return builder.none()
     fullname = expr.node.fullname
     if fullname in builtin_names:
         typ, src = builtin_names[fullname]
@@ -81,7 +88,7 @@ def transform_name_expr(builder: IRBuilder, expr: NameExpr) -> Value:
         if (isinstance(expr.node, Var) and is_none_rprimitive(builder.node_type(expr))
                 and expr.node.is_inferred):
             builder.error(
-                "Local variable '{}' has inferred type None; add an annotation".format(
+                'Local variable "{}" has inferred type None; add an annotation'.format(
                     expr.node.name),
                 expr.node.line)
 
@@ -95,7 +102,7 @@ def transform_name_expr(builder: IRBuilder, expr: NameExpr) -> Value:
             # instead load the module separately on each access.
             mod_dict = builder.call_c(get_module_dict_op, [], expr.line)
             obj = builder.call_c(dict_get_item_op,
-                                 [mod_dict, builder.load_static_unicode(expr.node.fullname)],
+                                 [mod_dict, builder.load_str(expr.node.fullname)],
                                  expr.line)
             return obj
         else:
@@ -124,7 +131,7 @@ def transform_member_expr(builder: IRBuilder, expr: MemberExpr) -> Value:
     if isinstance(typ, TupleType) and typ.partial_fallback.type.is_named_tuple:
         fields = typ.partial_fallback.type.metadata['namedtuple']['fields']
         if expr.name in fields:
-            index = builder.builder.load_static_int(fields.index(expr.name))
+            index = builder.builder.load_int(fields.index(expr.name))
             return builder.gen_method_call(obj, '__getitem__', [index], rtype, expr.line)
     return builder.builder.get_attr(obj, expr.name, rtype, expr.line)
 
@@ -189,10 +196,12 @@ def translate_refexpr_call(builder: IRBuilder, expr: CallExpr, callee: RefExpr)
     # they check that everything in arg_kinds is ARG_POS.
 
     # If there is a specializer for this function, try calling it.
+    # We would return the first successful one.
     if callee.fullname and (callee.fullname, None) in specializers:
-        val = specializers[callee.fullname, None](builder, expr, callee)
-        if val is not None:
-            return val
+        for specializer in specializers[callee.fullname, None]:
+            val = specializer(builder, expr, callee)
+            if val is not None:
+                return val
 
     # Gen the argument values
     arg_values = [builder.accept(arg) for arg in expr.args]
@@ -249,10 +258,12 @@ def translate_method_call(builder: IRBuilder, expr: CallExpr, callee: MemberExpr
         receiver_typ = builder.node_type(callee.expr)
 
         # If there is a specializer for this method name/type, try calling it.
+        # We would return the first successful one.
         if (callee.name, receiver_typ) in specializers:
-            val = specializers[callee.name, receiver_typ](builder, expr, callee)
-            if val is not None:
-                return val
+            for specializer in specializers[callee.name, receiver_typ]:
+                val = specializer(builder, expr, callee)
+                if val is not None:
+                    return val
 
         obj = builder.accept(callee.expr)
         args = [builder.accept(arg) for arg in expr.args]
@@ -383,13 +394,13 @@ def try_gen_slice_op(builder: IRBuilder, base: Value, index: SliceExpr) -> Optio
         if index.begin_index:
             begin = builder.accept(index.begin_index)
         else:
-            begin = builder.load_static_int(0)
+            begin = builder.load_int(0)
         if index.end_index:
             end = builder.accept(index.end_index)
         else:
             # Replace missing end index with the largest short integer
             # (a sequence can't be longer).
-            end = builder.load_static_int(MAX_SHORT_INT)
+            end = builder.load_int(MAX_SHORT_INT)
         candidates = [list_slice_op, tuple_slice_op, str_slice_op]
         return builder.builder.matching_call_c(candidates, [base, begin, end], index.line)
 
@@ -520,24 +531,24 @@ def transform_basic_comparison(builder: IRBuilder,
 
 
 def transform_int_expr(builder: IRBuilder, expr: IntExpr) -> Value:
-    return builder.builder.load_static_int(expr.value)
+    return builder.builder.load_int(expr.value)
 
 
 def transform_float_expr(builder: IRBuilder, expr: FloatExpr) -> Value:
-    return builder.builder.load_static_float(expr.value)
+    return builder.builder.load_float(expr.value)
 
 
 def transform_complex_expr(builder: IRBuilder, expr: ComplexExpr) -> Value:
-    return builder.builder.load_static_complex(expr.value)
+    return builder.builder.load_complex(expr.value)
 
 
 def transform_str_expr(builder: IRBuilder, expr: StrExpr) -> Value:
-    return builder.load_static_unicode(expr.value)
+    return builder.load_str(expr.value)
 
 
 def transform_bytes_expr(builder: IRBuilder, expr: BytesExpr) -> Value:
     value = bytes(expr.value, 'utf8').decode('unicode-escape').encode('raw-unicode-escape')
-    return builder.builder.load_static_bytes(value)
+    return builder.builder.load_bytes(value)
 
 
 def transform_ellipsis(builder: IRBuilder, o: EllipsisExpr) -> Value:
@@ -652,16 +663,7 @@ def transform_list_comprehension(builder: IRBuilder, o: ListComprehension) -> Va
 
 
 def transform_set_comprehension(builder: IRBuilder, o: SetComprehension) -> Value:
-    gen = o.generator
-    set_ops = builder.call_c(new_set_op, [], o.line)
-    loop_params = list(zip(gen.indices, gen.sequences, gen.condlists))
-
-    def gen_inner_stmts() -> None:
-        e = builder.accept(gen.left_expr)
-        builder.call_c(set_add_op, [set_ops, e], o.line)
-
-    comprehension_helper(builder, loop_params, gen_inner_stmts, o.line)
-    return set_ops
+    return translate_set_comprehension(builder, o.generator)
 
 
 def transform_dictionary_comprehension(builder: IRBuilder, o: DictionaryComprehension) -> Value:
